/*
 * Momentum Risk & Savings
 * Created on 2005/08/24
 *
 */

package za.co.momentum.recurring.base;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.zip.Adler32;
import java.util.zip.CheckedInputStream;
import java.util.zip.CheckedOutputStream;
import java.util.zip.Checksum;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;

import za.co.momentum.recurring.exceptions.DomainRuntimeException;

/**
 * @author Deon van Aarde
 * 
 */
public class Util {

  private static final Logger L = Logger.getLogger(Util.class);

  /**
   * These are escape sequences/symbols and their corresponding replacements used in the replaceEscapeSequences()
   * method.
   */
  private static final String[][] ESCAPE_SEQUENCES = new String[][] { { "&comma", "," }, { "&quot", "\"" } };

  /**
   * Empty string.
   */
  public static final String EMPTY_STRING = "";

  /**
   * Blank string ie. " "
   */
  public static final String BLANK_STRING = " ";

  /**
   * None string.
   */
  public static final String NONE_STRING = "None";

  /**
   * Buffer size for use in moving a file
   */
  public static final int BUFFER_SIZE = 4 * 1024;

  /**
   * Buffer size for use in zipping a file
   */
  public static final int ZIP_BUFFER_SIZE = 2 * 1024;

  /**
   * Zero string ie. "0"
   */
  public static final String ZERO_STRING = "0";

  // Boolean strings
  public static final String BOOLEAN_TRUE_STRING = "true";

  public static final String BOOLEAN_FALSE_STRING = "false";

  // Null object array.
  public static final Object[] NULL_OBJECT_ARRAY = new Object[] { null };

  // Empty arrays
  public static final String[] EMPTY_STRING_ARRAY = new String[0];

  // zero int[]
  public static final int[] ZERO_INT_ARRAY = new int[] { 0 };

  // zero double[]
  public static final double[] ZERO_DOUBLE_ARRAY = new double[] { 0 };

  /**
   * Dateformat instance for dates in the format yyyy-MM-dd.
   */
  private static DateFormat dateFormat;

  /**
   * Dateformat instance for dates in the format yyyy-MM-dd HH:mm.
   */
  private static DateFormat dateTimeFormat;

  /**
   * Dateformat instance for dates in the format yyyyMMdd.
   */
  private static DateFormat rawDateFormat;

  static {
    dateFormat = new SimpleDateFormat("yyyy-MM-dd");
    dateFormat.setLenient(false);
    dateTimeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
    rawDateFormat = new SimpleDateFormat("yyyyMMdd");
  }

  /**
   * Default rounding mechanism, as ROUND_UNNECESSARY is a timebomb that will explode the first time loss of precision
   * is detected.
   */
  public static int DEFAULT_ROUNDING = BigDecimal.ROUND_HALF_EVEN;

  private Util() {
  }

  public static String getExceptionStackAsString(Throwable ex) {
    ByteArrayOutputStream bytes = new ByteArrayOutputStream();
    PrintWriter writer = new PrintWriter(bytes, true);
    ex.printStackTrace(writer);
    return bytes.toString();
  }

  /**
   * Convenience method for centralising the printstack trace logic for exceptions in which we expect nested exceptions.
   * 
   * @param uniqueId
   *          uniqueId of t
   * @param nestedException
   *          this can be null, but if not, refers to the nestedException that may be contained in t.
   * @param nestedExceptionForPrint
   *          As nestedExceptions may contain further nestedExceptions, this specifies which one should be printed.
   */
  public static String getExceptionStackAsString(Throwable t, String uniqueId, Throwable nestedException,
      Throwable nestedExceptionForPrint) {
    ByteArrayOutputStream bytes = new ByteArrayOutputStream();
    PrintWriter writer = new PrintWriter(bytes, true);
    writer.println("\n********************************************************************");
    writer.println("*** Start of Exception (" + Util.getUnqualifiedClassName(t.getClass()) + ") with UniqueId: "
        + uniqueId);
    writer.println("*** Date: " + new Date());
    if (!Util.isEmpty(t.getMessage())) {
      writer.println("*** Message = " + t.getMessage());
    } else {
      writer.println("*** Message = EMPTY (ie. no message)");
    }

    if (nestedException != null) {
      writer.println("*** NestedException : Class = " + nestedException.getClass());
      nestedExceptionForPrint.printStackTrace(writer);
    } else {
      writer.println("*** NestedException : None.");
      writer.println("*** StackTrace = ");
      writer.println(Util.getExceptionStackAsString(t));
    }
    writer.println("*** ");
    writer.println("*** End   of Exception (" + Util.getUnqualifiedClassName(t.getClass()) + ") with UniqueId: "
        + uniqueId);
    writer.println("********************************************************************");
    return bytes.toString();
  }

  /**
   * Returns true if the passed string is null or "".
   * 
   * @param str
   * @return boolean true if null or ""
   */
  public static boolean isEmpty(String str) {
    return str == null || str.trim().equalsIgnoreCase(EMPTY_STRING);
  }

  /**
   * Inverse of isEmpty() for Strings.
   * 
   * @see isEmpty()
   */
  public static boolean isNotEmpty(String str) {
    return !Util.isEmpty(str);
  }

  /**
   * Inverse of isEmpty() for Collections
   * 
   * @see isEmpty()
   */
  public static boolean isNotEmpty(Collection coll) {
    return !Util.isEmpty(coll);
  }

  /**
   * Check if the integer value is empty
   * 
   * @param val
   * @return true if empty
   */
  public static boolean isEmpty(Integer val) {
    return val == null || val.intValue() == 0;
  }

  /**
   * Check if the BigDecimal value is empty
   * 
   * @param val
   * @return true if empty
   */
  public static boolean isEmpty(BigDecimal val) {
    return val == null || val.doubleValue() == 0;
  }

  /**
   * Returns true if the passed collection is null or has no objects in it ie. its size = 0.
   * 
   * @param coll
   * @return true if coll == null or coll.size() == 0.
   */
  public static boolean isEmpty(Collection coll) {
    return (coll == null) || (coll.isEmpty());
  }

  /**
   * Returns true if the given <tt>Object</tt> is null.
   */
  public static boolean isEmpty(Object o) {
    return (o == null);
  }

  /**
   * Pads the given <tt>str</tt> with <tt>padChar</tt> s until the combined length is <tt>lengthStr</tt>.<br>
   * Note that if <tt>lengthStr</tt> is >=<tt>str</tt>'s length, then <tt>str</tt> is returned if
   * <tt>forceLength</tt> is false.
   * 
   * @param str
   *          the string to pad.
   * @param padChar
   *          the char to pad with.
   * @param lengthStr
   *          the length of the returned string.
   * @param before
   *          if true, padding will be done before, otherwise after.
   * @param forceLength
   *          if false, then <tt>str</tt> will be returned if length of <tt>str</tt> is >=<tt>lengthStr</tt>.
   * @param forceLengthFromStart
   *          if true, then if <tt>forceLength</tt> is true and length of <tt>str</tt> is >=<tt>lengthStr</tt>,
   *          then the length of the returned string will be force to have length <tt>lengthStr</tt>, and this will
   *          be done by taking the characters from the start of the string, otherwise if this is false, then the new
   *          string will be created from the end of <tt>str</tt>.
   * @return String the new padded string.
   */
  public static String pad(String str, char padChar, int lengthStr, boolean padBefore, boolean forceLength,
      boolean forceLengthFromStart) {

    // first check is padding is required.
    if (str.length() >= lengthStr) {
      if (!forceLength)
        return str;
      if (forceLengthFromStart)
        return str.substring(0, lengthStr);
      return str.substring(str.length() - lengthStr - 1);
    }

    // build new padded string
    StringBuffer sbuf = new StringBuffer(padChar + "");
    for (int i = 0; i < (lengthStr - str.length() - 1); i++) {
      sbuf.append(padChar);
    }

    if (padBefore) { // append before
      return sbuf.append(str).toString();
    } else { // append after
      return str.concat(sbuf.toString());
    }
  }

  /**
   * Calls method pad with: <br>
   * padBefore = true; <br>
   * forceLength = "true"; <br>
   * forceLengthFromStart = true <br>
   * 
   * @see pad(String, char, int, boolean padBefore, boolean forceLength, boolean forceLengthFromStart )
   * @return String the new padded string.
   */
  public static String pad(String str, char padChar, int lengthStr) {
    return pad(str, padChar, lengthStr, true, true, true);
  }

  /**
   * Calls method pad with: <br>
   * forceLength = "true"; <br>
   * forceLengthFromStart = true <br>
   * 
   * @see padpad(String, char, int, boolean padBefore, boolean forceLength, boolean forceLengthFromStart )
   * @return String the new padded string.
   */
  public static String pad(String str, char padChar, int lengthStr, boolean padBefore) {
    return pad(str, padChar, lengthStr, padBefore, true, true);
  }

  /**
   * Returns the unqualified className for the given <tt>Class</tt> object. eg. if class = com.test.temp.MyClass, this
   * will return MyClass
   * 
   * @param className
   * @return String null if null is passed.
   */
  public static String getUnqualifiedClassName(Class clazz) {
    if (clazz == null) {
      return null;
    }
    StringTokenizer tkn = new StringTokenizer(clazz.getName(), ".");
    String result = "";
    while (tkn.hasMoreTokens()) {
      result = tkn.nextToken();
    }
    return result;
  }

  /**
   * Loads all properties defined in the passed file, and returns a <tt>Properties</tt> object.
   */
  public static Properties loadProperties(String propertiesFile) {
    try {
      File file = new File("path.tmp");
      try {
        Properties result = new Properties();
        InputStream is = new FileInputStream(propertiesFile);
        result.load(is);
        is.close();
        return result;
      } catch (FileNotFoundException fnfe) {
        file.createNewFile();
        throw new DomainRuntimeException("Property file not found in path " + file.getAbsoluteFile());
      } finally {
        file.delete();
      }
    } catch (Exception e) {
      throw new DomainRuntimeException("Unable to load " + propertiesFile, e);
    }
  }

  /**
   * This moves a file from one location to the next, deleting the original copy. It may happen from time to time that
   * deleting of the source file fails. This is something between Java and the host OS's file system. However, the file
   * will most likely then be deleted when the VM exits. In any case, if the source file wasn't successfully deleted,
   * this method will return false.
   * 
   * @param filename
   * @param fromDir
   * @param toDir
   * @return true if move successful
   */
  public static boolean moveFile(String filename, String fromDir, String toDir) {
    File sourceFile = new File(fromDir, filename);
    File destFile = new File(toDir, filename);
    L.debug("sourceFile=" + sourceFile.getAbsolutePath());
    L.debug("destFile=" + destFile.getAbsoluteFile());
    boolean success = false;
    FileInputStream reader = null;
    FileOutputStream writer = null;
    try {
      reader = new FileInputStream(sourceFile);
      writer = new FileOutputStream(destFile);
      if (!sourceFile.canRead())
        throw new DomainRuntimeException("Cannot read from " + sourceFile.getAbsoluteFile());
      if (!destFile.canWrite())
        throw new DomainRuntimeException("Cannot write to " + destFile.getAbsoluteFile());
      int len = 0;
      byte[] buffer = new byte[BUFFER_SIZE];
      while ((len = reader.read(buffer)) != -1)
        writer.write(buffer, 0, len);
    } catch (IOException ioe) {
      throw new DomainRuntimeException(ioe);
    } finally {
      try {
        if (reader != null)
          reader.close();
        if (writer != null) {
          writer.getFD().sync();
          writer.close();
        }
        if (!destFile.exists())
          throw new DomainRuntimeException("The destination was not created " + destFile.getAbsoluteFile());
      } catch (Exception e) {
        throw new DomainRuntimeException(e);
      }
      if (!sourceFile.delete()) {
        L.info("Unable to delete the source file " + sourceFile.getAbsoluteFile()
            + ". Trying to get it to delete on exit of the Java VM.");
        sourceFile.deleteOnExit();
      } else
        success = true;
    }
    return success;
  }

  /**
   * This copies a file from one location to the next, without overwriting the destination file if it exists.
   * 
   * @param sourceFilename
   * @param destFilename
   * @return true if copy successful
   */
  public static boolean copyFile(String sourceFilename, String destFilename) {
    File sourceFile = new File(sourceFilename);
    File destFile = new File(destFilename);
    L.debug("sourceFile=" + sourceFile.getAbsolutePath());
    L.debug("destFile=" + destFile.getAbsoluteFile());
    if (destFile.exists())
      throw new DomainRuntimeException("Cannot copy file, destination already exists: " + destFile.getAbsolutePath());
    boolean success = false;
    FileInputStream reader = null;
    FileOutputStream writer = null;
    try {
      reader = new FileInputStream(sourceFile);
      writer = new FileOutputStream(destFile);
      if (!sourceFile.canRead())
        throw new DomainRuntimeException("Cannot read from " + sourceFile.getAbsoluteFile());
      if (!destFile.canWrite())
        throw new DomainRuntimeException("Cannot write to " + destFile.getAbsoluteFile());
      int len = 0;
      byte[] buffer = new byte[BUFFER_SIZE];
      while ((len = reader.read(buffer)) != -1)
        writer.write(buffer, 0, len);
    } catch (IOException ioe) {
      throw new DomainRuntimeException(ioe);
    } finally {
      try {
        if (reader != null)
          reader.close();
        if (writer != null) {
          writer.getFD().sync();
          writer.close();
        }
        if (!destFile.exists())
          throw new DomainRuntimeException("The destination was not created " + destFile.getAbsoluteFile());
      } catch (Exception e) {
        throw new DomainRuntimeException(e);
      }
    }
    return success;
  }

  /**
   * This deletes a file
   * 
   * @param sourceFilename
   * @return true if delete successful
   */
  public static boolean deleteFile(String sourceFilename) {
    File sourceFile = new File(sourceFilename);
    return sourceFile.delete();
  }

  /**
   * Returns true if the passed char is a letter - uppercase or lowercase. A letter is defined as any character that is
   * part of the standard alphabet ie 'a' to 'z'
   */
  public static boolean isLetter(char ch) {
    int type = Character.getType(ch);
    if (type == Character.UPPERCASE_LETTER || type == Character.LOWERCASE_LETTER) {
      return true;
    }

    return false;
  }

  /**
   * Iterates through each character of the given string, and returns true if all the characters are letters.
   * 
   * @see isLetter(char)
   * @return
   */
  public static boolean isAllLetters(String str) {
    for (int i = 0; i < str.length(); i++) {
      if (!isLetter(str.charAt(i))) {
        return false;
      }
    }
    return true;
  }

  /**
   * Returns true if <tt>date</tt> is >=<tt>date1</tt> and <tt>date2</tt>
   * 
   * @return boolean
   */
  public static boolean isBetween(Date date, Date date1, Date date2) {
    if (date.equals(date1) || date.equals(date2)) {
      return true;
    }

    if (date.after(date1) && date.before(date2)) {
      return true;
    }

    return false;
  }

  /**
   * Returns true if the two passed collections are equal ie. they contain the same objects (when comparing with their
   * equals() method). Note that this method differs from the collection equals() method, in that sequence is not
   * important ie. it does not matter if object A is in position 1 of colla, and in position 2 of collb.
   * 
   * @param colla
   * @param collb
   * @return boolean.
   */
  public static boolean collectionsAreEqual(Collection colla, Collection collb) {
    // first, compare sizes
    if (colla.size() != collb.size()) {
      return false;
    }

    // check if colla fully contains collb and collb fully contains colla.
    if (colla.containsAll(collb) && collb.containsAll(colla)) {
      return true;
    }

    return false;
  }

  /**
   * Returns a <tt>BufferedReader</tt> for the given file. The given file must be its fully qualified filename. Note
   * that the reader returned is positioned at the second line, as the first line is a comment on the structure of the
   * file.
   */
  public static BufferedReader getDataReader(String fileName) throws FileNotFoundException {
    try {
      BufferedReader reader = new BufferedReader(new FileReader(fileName));
      // Ignore first line.
      // reader.readLine();
      return reader;
    } catch (IOException ex) {
      throw new DomainRuntimeException("Error in trying to read file : " + fileName, ex);
    }
  }

  /**
   * Returns a <tt>BufferedWriter</tt> for the given file. The given file must be its fully qualified filename.
   */
  public static BufferedWriter getDataWriter(String fileName) {
    try {
      BufferedWriter writer = new BufferedWriter(new FileWriter(fileName));
      return writer;
    } catch (IOException ex) {
      throw new DomainRuntimeException("Error in trying to create writer to file : " + fileName, ex);
    }
  }

  /**
   * Returns the unqualified className from the given FULL className.
   * 
   * @param className
   * @return String null if null is passed.
   */
  public static String getUnqualifiedClassName(String fullClassName) {
    if (fullClassName == null) {
      return null;
    }
    StringTokenizer tkn = new StringTokenizer(fullClassName, ".");
    String result = "";
    while (tkn.hasMoreTokens()) {
      result = tkn.nextToken();
    }
    return result;
  }

  /**
   * Returns the package name from the given fully qualified class name.
   * 
   * @param className
   * @return String null if className == null.
   */
  public static String getPackageName(String className) {
    if (className == null) {
      return null;
    }

    StringTokenizer tkn = new StringTokenizer(className, ".");
    String result = tkn.nextToken();
    String currentToken = null;
    while (tkn.hasMoreTokens()) {
      currentToken = tkn.nextToken();
      if (tkn.hasMoreTokens()) {
        result += ("." + currentToken);
      }
    }
    return result;
  }

  /**
   * Returns the package name from the given <code>Class</code> object.
   * 
   * @param clazz
   * @return String null if className == null.
   */
  public static String getPackageName(Class clazz) {
    return getPackageName(clazz.getName());
  }

  /**
   * Parses <tt>fileRef</tt> which is structured as follows: <br>
   * fileName{ref1:ref2:ref3:...etc}. <br>
   * fileName is a fully qualified filename, where the file is structured as :<br>
   * ref <i>x </i>, dataField, dataField, etc; ,<br>
   * where x=1,2,3 etc
   * 
   * @param fileRef
   * @return An ArrayList of String objects where each string object is a line in the given file that corresponds to one
   *         of the given references.
   */
  public static ArrayList getListofRelationships(String fileRef) {
    ArrayList result = new ArrayList();
    StringTokenizer stkn = new StringTokenizer(fileRef, "{");
    String fileName = stkn.nextToken().trim();
    StringTokenizer stkn1 = new StringTokenizer(stkn.nextToken("}"), ":");
    Set keys = new HashSet();
    while (stkn1.hasMoreTokens()) {
      keys.add(stkn1.nextToken().trim());
    }
    try {
      BufferedReader reader = getDataReader(fileName);
      String line = null;
      StringTokenizer stkn3 = null;
      String id = null;
      while ((line = reader.readLine()) != null) {
        stkn3 = new StringTokenizer(line, ",");
        id = stkn3.nextToken().trim();
        if (keys.contains(id)) {
          result.add(line);
        }
      }
    } catch (IOException ex) {
      throw new DomainRuntimeException("Error parsing file:" + fileName, ex);
    }

    return result;
  }

  /**
   * Generates a large primitive longg.
   */
  public static long genRandomPrimitiveLong() {
    return System.currentTimeMillis();
  }

  /**
   * Generates a large primitive int
   */
  public static int genRandomPrimitiveInt() {
    Calendar cal = new GregorianCalendar(1990, 01, 01);
    cal.getTime().getTime();
    return (int) (System.currentTimeMillis() - cal.getTime().getTime());
  }

  /**
   * Generates a primitive int between 0 and the given upper bound.
   */
  public static int genRandomPrimitiveInt(int upperBound) {
    return (int) (Math.random() * upperBound);
  }

  /**
   * Ensures consistent rounding use throughout the system
   * 
   * @return int
   * @deprecated see NumberUtil
   */
  public static int getDefaultRoundingMode() {
    return BigDecimal.ROUND_HALF_EVEN;
  }

  /**
   * Ensures the same scale is used consistently throughout
   * 
   * @return int
   * @deprecated see NumberUtil
   */
  public static int getDefaultScale() {
    return 2;
  }

  /**
   * Generates a large primitive.
   */
  public static Integer genRandomInteger() {
    Calendar cal = new GregorianCalendar(1990, 01, 01);
    cal.getTime().getTime();
    return new Integer((int) (System.currentTimeMillis() - cal.getTime().getTime()));
  }

  /**
   * Generates a random Integer greater than lowValue
   */
  public static Integer genRandomInteger(int lowValue) {
    double val1 = Math.random();
    double val2 = Math.random();
    int val = (int) (val1 * val2 * lowValue);
    return new Integer(lowValue + val);
  }

  /**
   * Returns the given double as an amount ie. with 2 decimal precision rounded.
   */
  public static double roundToAmount(double number) {
    return (Math.round(number * 100.00)) / 100.00;
  }

  /**
   * Handle the situation where the string might be null
   * 
   * @param s
   * @return String (trimmed)
   */
  public static String trim(String s) {
    if (Util.isEmpty(s))
      return s;
    else
      return s.trim();
  }

  /**
   * Handle the situation where the string might be null
   * 
   * @param s
   * @return String (trimmed)
   */
  public static String trim(String s, char c) {
    if (Util.isEmpty(s))
      return s;
    char[] theChars = s.toCharArray();
    StringBuffer stripped = new StringBuffer();
    for (int i = 0; i < theChars.length; i++) {
      if (theChars[i] != c) {
        stripped.append(theChars[i]);
      }
    }
    return stripped.toString();
  }

  /**
   * Truncate the string to some specified length
   * 
   * @param s
   * @param length
   * @return The truncated string
   */
  public static String truncate(String s, int length) {
    if (length < 0)
      throw new DomainRuntimeException("Cannot truncate to length: " + length);
    if (length > s.length())
      return s;
    else
      return s.substring(0, length);
  }

  /**
   * Takes a string and strips a character from the front (pre is true) or the back (pre is false).
   * 
   * @param s
   *          The string to process
   * @param c
   *          The character to remove
   * @param pre
   *          From the front if true, from the back otherwise.
   * @return The stripped string
   */
  public static String strip(String s, char c, boolean pre) {
    char[] theChars = s.toCharArray();
    StringBuffer stripped = new StringBuffer();
    if (pre) {
      for (int i = 0; i < theChars.length; i++) {
        if (theChars[i] != c) {
          stripped.append(theChars, i, theChars.length - i);
          break;
        }
      }
    } else {
      for (int i = theChars.length - 1; i > -1; i--) {
        if (theChars[i] != c) {
          stripped.append(theChars, 0, i + 1);
          break;
        }
      }
    }
    return stripped.toString();
  }

  /**
   * Converts a timestamp millis to a fractional millis.
   */
  public static double getTimestampinMillis(java.sql.Timestamp timestamp) {
    double result = timestamp.getTime(); // excluses nanoseconds
    result += (timestamp.getNanos() / 1000000.);
    return result;
  }

  /**
   * Returns t1 - t2 in seconds (-ve is returned if t2 > t1).
   */
  public static BigDecimal getTimestampDiffInSecs(java.sql.Timestamp t1, java.sql.Timestamp t2) {
    double dt1 = getTimestampinMillis(t1);
    double dt2 = getTimestampinMillis(t2);
    BigDecimal result = new BigDecimal((dt1 - dt2) / 1000.00);
    return result.setScale(3, BigDecimal.ROUND_HALF_UP);
  }

  /**
   * Takes a comma-separated-values string and pop the values into a collection
   * 
   * @param csvString
   * @return Collection of string values
   */
  public static Collection getCollectionFromCSVString(String csvString) {
    Collection values = new ArrayList();
    if (Util.isEmpty(csvString))
      return values;
    for (StringTokenizer tokenizer = new StringTokenizer(csvString, ","); tokenizer.hasMoreTokens();) {
      values.add(tokenizer.nextToken().trim().intern());
    }
    return values;
  }

  /**
   * Returns a java.sql.Date reprenting the current date with no time element at all.
   */
  public static java.sql.Date getCurrentDateWithNoTime() {
    return getDateWithNoTime(new Date(System.currentTimeMillis()));
  }

  /**
   * Returns a long representing the current time in millis.
   * 
   * @see System#currentTimeInMillis.
   */
  public static long getCurrentTimeInMillis() {
    return System.currentTimeMillis();
  }

  /**
   * Returns a java.sql.Timestamp representing the current date and time.
   */
  public static java.sql.Timestamp getCurrentTimestamp() {
    return new java.sql.Timestamp(System.currentTimeMillis());
  }

  /**
   * Returns the time lapsed in ms.
   */
  public static long getTimelapse(long startTimeMillis) {
    return getCurrentTimeInMillis() - startTimeMillis;
  }

  /**
   * Converts a java.util.Date into a java.sql.Date with zero millisecond.
   */
  public static java.sql.Date getDateWithNoTime(java.util.Date dateTime) {
    Calendar cal = new GregorianCalendar();
    cal.setTime(dateTime);
    Calendar calWithNoTime = new GregorianCalendar(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal
        .get(Calendar.DATE));

    return new java.sql.Date(calWithNoTime.getTime().getTime());
  }

  /**
   * Instantiates and returns an Integer object from the given string, which should be a number.
   * 
   * @throws NumberFormatException
   *           if numStr is not a number.
   */
  public static Integer getInteger(String numStr) {
    return new Integer(numStr.trim());
  }

  /**
   * Instantiates and returns a BigDecimal object from the given int.
   * 
   * @throws NumberFormatException
   *           if numStr is not a number.
   */
  public static BigDecimal getBigDecimal(int num) {
    return getBigDecimal(String.valueOf(num));
  }

  /**
   * Instantiates and returns a BigDecimal object from the given string, which should be a number.
   * 
   * @throws NumberFormatException
   *           if numStr is not a number.
   */
  public static BigDecimal getBigDecimal(String numStr) {
    return new BigDecimal(numStr.trim());
  }

  /**
   * Returns a Boolean object corresponding to the passed primitive boolean.
   */
  public static Boolean getBoolean(boolean bool) {
    return Boolean.valueOf(bool + EMPTY_STRING);
  }

  /**
   * Returns primitive boolean true or false based on the passed string, which should only be "true" or "false".
   * 
   * @param booleanStr
   * @return
   */
  public static boolean getBooleanPrimitive(String booleanStr) {
    if (booleanStr.trim().equalsIgnoreCase("true") || booleanStr.trim().equals("1")) {
      return Boolean.TRUE.booleanValue();
    } else if (booleanStr.trim().equalsIgnoreCase("false") || booleanStr.trim().equals("0")) {
      return Boolean.FALSE.booleanValue();
    } else {
      throw new DomainRuntimeException("Error: Invalid boolean value - " + booleanStr
          + ". Should only be true or false.");
    }
  }

  /**
   * Returns a primitive double from the given string, which should be a number.
   * 
   * @throws NumberFormatException
   *           if doubleStr is not a number.
   */
  public static double getDoublePrimitive(String doubleStr) {
    return Double.parseDouble(doubleStr.trim());
  }

  /**
   * Returns a primitive int from the given string, which should be a number.
   * 
   * @throws NumberFormatException
   *           if intStr is not a number.
   */
  public static int getIntPrimitive(String intStr) {
    return Integer.parseInt(intStr.trim());
  }

  /**
   * Returns a int primitive from the given object, which must be a int[] with size == 1.
   * 
   * @return obj[0].
   * @throws DomainRuntimeException
   *           if obj is not an instance of int[] or obj.size != 1.
   */
  public static int getIntPrimitiveFromObject(Object obj) {
    if (!(obj instanceof int[])) {
      throw new DomainRuntimeException("Error: Object must be an int[]");
    }
    int[] i = (int[]) obj;
    if (i == null || i.length != 1) {
      throw new DomainRuntimeException("Error: int[] size must = 1.");
    }
    return i[0];
  }

  /**
   * Returns a double primitive from the given object, which must be a double[] with size == 1.
   * 
   * @return obj[0].
   * @throws DomainRuntimeException
   *           if obj is not an instance of double[] or obj.size != 1.
   */
  public static double getDoublePrimitiveFromObject(Object obj) {
    if (!(obj instanceof double[])) {
      throw new DomainRuntimeException("Error: Object must be an double[]");
    }
    double[] i = (double[]) obj;
    if (i == null || i.length != 1) {
      throw new DomainRuntimeException("Error: double[] size must = 1.");
    }
    return i[0];
  }

  /**
   * Returns a boolean primitive from the given object, which must be a boolean[] with size == 1.
   * 
   * @return obj[0].
   * @throws DomainRuntimeException
   *           if obj is not an instance of boolean[] or obj.size != 1.
   */
  public static boolean getBooleanPrimitiveFromObject(Object obj) {
    if (!(obj instanceof boolean[])) {
      throw new DomainRuntimeException("Error: Object must be an boolean[]");
    }
    boolean[] i = (boolean[]) obj;
    if (i == null || i.length != 1) {
      throw new DomainRuntimeException("Error: boolean[] size must = 1.");
    }
    return i[0];
  }

  /**
   * Returns a string version of given boolean.
   */
  public static String getStringFromBoolean(boolean bool) {
    if (bool) {
      return BOOLEAN_TRUE_STRING;
    }
    return BOOLEAN_FALSE_STRING;
  }

  /**
   * Takes a date and returns a string formated. Converse of getDate().
   * 
   * @param date
   * @return String
   */
  public static String getStringFromDate(java.util.Date date) {
    return dateFormat.format(date);
  }

  public static String getStringFromDate(java.sql.Date date) {
    return getStringFromDate(new java.util.Date(date.getTime()));
  }

  /**
   * Takes a Calendar returns a string format of the date in the calendar.
   * 
   * @return String in format dd/mm/yyyy.
   */
  public static String getStringFromDate(Calendar cal) {
    return dateFormat.format(cal.getTime());
  }

  /**
   * Takes a timestamp and returns a string formated. Converse of getTimestamp().
   * 
   * @param date
   * @return String
   */
  public static String getStringFromTimestamp(java.sql.Timestamp timestamp) {
    return dateTimeFormat.format(timestamp);
  }

  /**
   * Takes a date and returns a string formated.
   * 
   * @param date
   * @return String
   */

  public static String formatDateToDateTime(java.util.Date date) {
    return dateTimeFormat.format(date);
  }

  /**
   * Takes a long and returns a string formated.
   * 
   * @param long
   * @return String
   */

  public static String formatLongToDate(long dateInMillisecs) {
    Date d = new Date(dateInMillisecs);
    return dateFormat.format(d);
  }

  /**
   * Takes in a standard java name, capitalises the first letter, and separates out each subsequent word that starts
   * with a capital letter.
   */
  public static String getAttributeLabel(String name) {
    String firstChar = name.substring(0, 1).toUpperCase();
    char[] chArray = name.substring(1).toCharArray();
    StringBuffer word = new StringBuffer(firstChar);
    String result = "";
    String sep = Util.EMPTY_STRING;
    for (int i = 0; i < chArray.length; i++) {
      if (Character.isUpperCase(chArray[i])) {
        result += (sep + word.toString());
        word = new StringBuffer(chArray[i] + Util.EMPTY_STRING);
        sep = Util.BLANK_STRING;
      } else {
        word.append(chArray[i]);
      }
    }
    result += (sep + word.toString());
    return result;
  }

  /**
   * Returns a java.sql.Date from the given string, which must be in format dd/mm/yyyy. This method assumes that the
   * given date is correct, otherwise it throws a <tt>DomainRuntimeException</tt>
   * 
   * @throws DomainRuntimeException
   *           if date is invalid. Used method <tt>isDateValid</tt> to check date validity first.
   */
  public static java.sql.Date getDate(String date) {
    if (date == null) {
      return null;
    }
    try {
      date = date.trim();
      if (date.startsWith("/") || date.endsWith("/") || date.indexOf("//") != -1) {
        throw new DomainRuntimeException("Invalid date: " + date);
      }
      Date temp = dateFormat.parse(date);
      return Util.getDateWithNoTime(temp);
    } catch (ParseException ex) {
      throw new DomainRuntimeException("Invalid date:" + date, ex);
    }
  }

  /**
   * The date from the AS400 is in the format ccyymmdd
   * 
   * @param date
   * @return
   */
  public static java.sql.Date getDateFromAS400(String date) {
    if (date == null) {
      return null;
    }
    try {
      Date temp = rawDateFormat.parse(date);
      return Util.getDateWithNoTime(temp);
    } catch (ParseException ex) {
      throw new DomainRuntimeException("Invalid date:" + date, ex);
    }
  }

  /**
   * The date on the AS400 is in the format ccyymmdd
   * 
   * @param date
   * @return
   */
  public static BigDecimal setAS400Date(java.sql.Date date) {
    if (date == null) {
      return null;
    }
    try {
      BigDecimal temp = new BigDecimal(rawDateFormat.format(date));
      return temp;
    } catch (Exception e) {
      throw new DomainRuntimeException("Invalid date format:" + date, e);
    }
  }

  /**
   * Returns a Calendar from the given date. Needed for xml parsing using JAXB
   * 
   * @param date
   * @return Calendar
   */
  public static Calendar getDate(java.sql.Date date) {
    if (date == null) {
      return null;
    }
    Calendar calendar = new GregorianCalendar();
    calendar.setTime(date);
    return calendar;
  }

  /**
   * Returns a java.sql.Timestamp from the given string, which must be in format dd/MM/yyyy hh:mm aaa. This method
   * assumes that the given date is correct, otherwise it throws a <tt>DomainRuntimeException</tt>
   * 
   * @throws DomainRuntimeException
   *           if date is invalid. Used method <tt>isDateValid</tt> to check date validity first.
   */
  public static java.sql.Timestamp getTimestamp(String ts) {
    if (ts == null) {
      return null;
    }
    try {
      Date temp = dateTimeFormat.parse(ts.trim());
      return new java.sql.Timestamp(temp.getTime());
    } catch (ParseException ex) {
      throw new DomainRuntimeException("Invalid date and time. Format expected: dd/MM/yyyy hh:mm aaa", ex);
    }
  }

  /**
   * Returns true if the given date is a valid calendar date.
   * 
   * @return true if valid calendar date. Note that if date = null, then false is returned.
   */
  public static boolean isValidDate(String date) {
    if (date == null) {
      return false;
    }
    try {
      dateFormat.parse(date);
    } catch (ParseException ex) {
      return false;
    }
    return true;
  }

  /**
   * Returns true if the passed string is a valid amount. A valid amount is effectively a double (which could have comma
   * separated numbers) + a max of 2 decimals. It is also greater than or equal to zero.
   */
  public static boolean isValidAmount(String amount) {
    try {
      Number num = NumberFormat.getNumberInstance().parse(amount);
      double result = num.doubleValue();
      if (result < 0)
        return false;
      result = num.doubleValue() * 100;
      int temp = (int) result;
      if (result - temp != 0.0) {
        return false;
      }
      return true;
    } catch (ParseException ex) {
      return false;
    }
  }

  /**
   * Gets amount for the passed string - which is assumed to be a correctly formatted string amount.
   * 
   * @see isValidAmount
   * @throws DomainRuntimeException
   *           in the evnt that amount is not valid.
   */
  public static double getAmount(String amount) {
    try {
      Number num = NumberFormat.getNumberInstance().parse(amount);
      double result = num.doubleValue() * 100;
      int temp = (int) result;
      if (result - temp != 0.0) {
        throw new DomainRuntimeException("Error: invalid amount - " + amount + ".");
      }
      return num.doubleValue();
    } catch (ParseException ex) {
      throw new DomainRuntimeException("Error: invalid amount - " + amount + ".");
    }
  }

  /**
   * Parses the given string by replacing all _ with " " and returning the resulting string.
   * 
   * @returns empty string if label = null or empty.
   */

  public static String getDisplayLabel(String label) {
    if (Util.isEmpty(label))
      return Util.EMPTY_STRING;
    StringTokenizer tkn = new StringTokenizer(label, "_");
    StringBuffer result = new StringBuffer();
    while (tkn.hasMoreTokens()) {
      result.append(tkn.nextToken() + Util.BLANK_STRING);
    }
    return result.toString();
  }

  /**
   * Parses out any '_' chars, converts each character after each '_' char to an uppercase, and returns the result. This
   * is often used for parsing strings to return standard java names. Note that the first char is left alone, which
   * effectively means that this parsing mechanism can be used for Java variable and method names.
   */
  public static String parseStandardName(String str) {
    StringTokenizer tkn = new StringTokenizer(str, "_");
    StringBuffer result = new StringBuffer(tkn.nextToken());
    String word = null;
    while (tkn.hasMoreTokens()) {
      word = tkn.nextToken();
      result.append(word.substring(0, 1).toUpperCase() + word.substring(1));
    }
    return result.toString();
  }

  /**
   * Converts the given collection to an arraylist. Note, currently this method assumes that the collection is an
   * arraylist.
   */
  public static ArrayList convertCollectionToArrayList(Collection coll) {
    return (ArrayList) coll;
  }

  /**
   * Provides a central mechanism to give a standard message in the event of a finder exception.
   */
  public static String getFinderMessage(String entityName, Object entityId) {
    if (entityId instanceof Integer) {
      return "Error occurred while trying to find object of type : " + entityName + " with identifier: id = "
          + entityId;
    } else {
      return "Error occurred while trying to find object of type : " + entityName + " with identifier: " + entityId;
    }
  }

  /**
   * Returns true if the given number is in the given range (inclusive).
   */
  public static boolean isInRange(double num, double lower, double upper) {
    return (num >= lower && num <= upper);
  }

  /**
   * Returns a full path file name, combining the dir and file name.
   * 
   * @param dir
   *          this can end with a / or not.
   */
  public static String constructFileName(String dir, String fileName) {
    return dir.trim() + "/" + fileName.trim();
  }

  /**
   * Takes in two collections. Objects of part collection must be fully contained by full collection. Note that the
   * objects inside he collections must properly implements the equals method.
   * 
   * @param full
   *          can be any Object coll
   * @param part
   *          can be any Object coll
   * @return true if part is fully contained in full.
   * @return true if part is null or empty - it is assumed that this means it is fully contained.
   */
  public static boolean containsAllObjects(Collection full, Collection part) {
    if (Util.isEmpty(part))
      return true;
    for (Iterator i = part.iterator(); i.hasNext();) {
      if (!full.contains(i.next())) {
        return false;
      }
    }
    return true;
  }

  /**
   * Returns the given attribute name as a Java accessor name eg. if sent "orderNo", then "getOrderNo" is returned.
   */
  public static String getJavaAccessorName(String attrName) {
    return "get" + attrName.substring(0, 1).toUpperCase() + attrName.substring(1);
  }

  /**
   * Returns the given attribute name as a Java mutator name eg. if sent "orderNo", then "setOrderNo" is returned.
   */
  public static String getJavaMutatorName(String attrName) {
    return "set" + attrName.substring(0, 1).toUpperCase() + attrName.substring(1);
  }

  /**
   * Returns true if n1 == n2 in terms of value.
   * 
   * @deprecated see NumberUtil
   */
  public static boolean isEqual(BigDecimal n1, BigDecimal n2) {
    return n1.compareTo(n2) == 0;
  }

  /**
   * Returns true if n1 > n2 in terms of value.
   * 
   * @deprecated see NumberUtil
   */
  public static boolean isGreaterThan(BigDecimal n1, BigDecimal n2) {
    return n1.compareTo(n2) > 0;
  }

  /**
   * Returns true if n1 < n2 in terms of value.
   * 
   * @deprecated see NumberUtil
   */
  public static boolean isLessThan(BigDecimal n1, BigDecimal n2) {
    return n1.compareTo(n2) < 0;
  }

  /**
   * Returns true if n1 <= n2 in terms of value.
   * 
   * @deprecated see NumberUtil
   */
  public static boolean isLessThanOrEqualTo(BigDecimal n1, BigDecimal n2) {
    return n1.compareTo(n2) < 1;
  }

  /**
   * Returns the next token (trimmed) in the given tokenizer.
   */
  public static String getNextToken(StringTokenizer stkn) {
    return stkn.nextToken().trim();
  }

  /**
   * Returns an integer array from the given int[]
   * 
   * @param intArray,
   *          if null, then null is returned.
   */
  public static Integer[] getIntegerArray(int[] intArray) {
    if (intArray == null) {
      return null;
    }

    Integer[] integerArray = new Integer[intArray.length];
    for (int i = 0; i < intArray.length; i++) {
      integerArray[i] = new Integer(intArray[i]);
    }
    return integerArray;
  }

  /**
   * Returns the given props as a String, within which each property is alphabetically sorted. Useful for printing out
   * the given props object.
   */
  public static String getPropertiesAsAlphabeticString(Properties props) {
    if (props == null) {
      return null;
    }
    Enumeration enumerator = props.keys();
    String result = Util.EMPTY_STRING;
    String key = null;
    Set sorted = new TreeSet();
    for (; enumerator.hasMoreElements();) {
      key = (String) enumerator.nextElement();
      sorted.add(key + "=" + props.getProperty(key));
    }
    for (Iterator i = sorted.iterator(); i.hasNext();) {
      result += (i.next() + " ***\r\n");
    }
    return result;
  }

  /**
   * Returns the ip address of the given hostname.
   * 
   * @return String = ipaddress in dot notation.
   */
  public static String getIPAddressOfHostName(String hostName) throws UnknownHostException {
    InetAddress addr = InetAddress.getByName(hostName);
    byte[] ipAddr = addr.getAddress();

    // Convert to dot representation
    String ipAddrStr = EMPTY_STRING;
    for (int i = 0; i < ipAddr.length; i++) {
      if (i > 0) {
        ipAddrStr += ".";
      }
      ipAddrStr += ipAddr[i] & 0xFF;
    }
    return ipAddrStr;
  }

  /**
   * Returns the IP address of the local host machine in dot notation.
   */
  public static String getIPAddressOfLocalHost() throws UnknownHostException {
    return getIPAddressOfHostName("localhost");
  }

  /**
   * Returns the full file name. Assumes any separators use the Unix "/" format.
   */
  public static String getFileName(String dir, String shortFileName) {
    if (dir.endsWith("/")) {
      return dir + shortFileName;
    } else {
      return dir + "/" + shortFileName;
    }
  }

  /**
   * Returns the given file as a string, using the default fileSep = "\r\n";
   * 
   * @param fileName
   *          full absolute path name for file.
   */
  public static String getFileAsString(String fileName) {
    return getFileAsString(fileName, "\r\n");
  }

  /**
   * Returns the given file as a string, using the given fileSep string as a separator.
   * 
   * @param fileName
   *          full absolute path name for file.
   */
  public static String getFileAsString(String fileName, String fileSep) {
    try {
      BufferedReader in = new BufferedReader(new FileReader(fileName));
      String str = null;
      StringBuffer result = new StringBuffer();
      while ((str = in.readLine()) != null) {
        result.append(str + fileSep);
      }
      return result.toString();
    } catch (Exception ex) {
      throw new DomainRuntimeException(ex);
    }
  }

  /**
   * Reads the given file and returns a collection of strings where each string ends with delimiter char in the file.
   * 
   * @param replaceStrings
   *          contains key/value pairs means replace key with value in the resulting collection of strings.
   * @return List of Strings.
   */
  public static List getFileAsCollectionOfStrings(String fileName, String delimiter, Map replaceStrings) {
    StringBuffer sb = new StringBuffer();
    // List list = new ArrayList();
    try {
      BufferedReader in = new BufferedReader(new FileReader(fileName));
      String str = null;
      while ((str = in.readLine()) != null) {
        sb.append(str);
      }
      return getStringAsCollectionOfStrings(sb.toString(), delimiter, replaceStrings);
    } catch (Exception ex) {
      throw new DomainRuntimeException(Util.getExceptionStackAsString(ex));
    }
  }

  /**
   * Reads the given InputStream and returns a collection of strings where each string ends with delimiter char in the
   * file.
   * 
   * @param replaceStrings
   *          contains key/value pairs means replace key with value in the resulting collection of strings.
   * @return List of Strings.
   */
  public static List getStreamAsCollectionOfStrings(InputStream stream, String delimiter, Map replaceStrings) {
    StringBuffer sb = new StringBuffer();
    // List list = new ArrayList();
    try {
      BufferedReader in = new BufferedReader(new InputStreamReader(stream));
      String str = null;
      while ((str = in.readLine()) != null) {
        sb.append(str);
      }
      return getStringAsCollectionOfStrings(sb.toString(), delimiter, replaceStrings);
    } catch (Exception ex) {
      throw new DomainRuntimeException(Util.getExceptionStackAsString(ex));
    }
  }

  /**
   * Reads the given InputStream and returns it as a String
   * 
   * @return String
   */
  public static String getStreamString(InputStream stream) {
    StringBuffer sb = new StringBuffer();
    // List list = new ArrayList();
    try {
      BufferedReader in = new BufferedReader(new InputStreamReader(stream));
      // String str = null;
      while (in.ready()) {
        char c = (char) in.read();
        if (c == '\n' || c == '\r')
          sb.append(' ');
        else
          sb.append(c);
      }
      return sb.toString();
    } catch (Exception ex) {
      throw new DomainRuntimeException(Util.getExceptionStackAsString(ex));
    }
  }

  /**
   * Reads the given String returns a collection of strings where each string ends with delimiter char in the given
   * string.
   * 
   * @parame replaceStrings - see getFileAsCollectionOfStrings
   * @return List of Strings.
   */
  public static List getStringAsCollectionOfStrings(String str, String delimiter, Map replaceStrings) {
    List list = new ArrayList();
    String parsedString = str;
    Map.Entry entry = null;
    for (Iterator i = replaceStrings.entrySet().iterator(); i.hasNext();) {
      entry = (Map.Entry) i.next();
      parsedString = replaceString(parsedString, (String) entry.getKey(), (String) entry.getValue());
    }
    StringTokenizer tkn = new StringTokenizer(parsedString, delimiter);
    String result = null;
    while (tkn.hasMoreTokens()) {
      result = tkn.nextToken();
      if (result != null && result.trim().length() > 0)
        list.add(result);
    }
    return list;
  }

  /**
   * Return the directory from the given file name. The directory is assumed to be the string prior to the last "/".
   */
  public static String getDirectoryFromFileName(String fileName) {
    int lastIndex = fileName.lastIndexOf("/");
    return fileName.substring(0, lastIndex);
  }

  /**
   * Returns the short file name ie. without the directory name. fileName is assumed to have "/" or "\" as the file
   * separator.
   * 
   * @return null if fileName is null.
   */
  public static String getFileShortName(String fileName) {

    if (fileName == null) {
      return null;
    }
    if (fileName.indexOf("/") != -1)
      return fileName.substring(fileName.lastIndexOf("/"));
    else {
      if (fileName.indexOf("\\") != -1) {
        return fileName.substring(fileName.lastIndexOf("\\"));
      }
    }
    return fileName;
  }

  /**
   * Deletes all files in dir and subdirs.
   */
  public static void deleteAllFiles(File f) {
    try {
      File[] files = f.listFiles();
      if (files == null)
        return;
      for (int i = 0; i < files.length; i++) {
        if (files[i].isFile()) {
          files[i].delete();
        } else {
          deleteAllFiles(files[i]);
          files[i].delete();
        }
      }
    } catch (Exception ex) {
      throw new DomainRuntimeException(getExceptionStackAsString(ex));
    }
  }

  // ****************************************************************************
  // BELOW ADDED BY GOPAL - @todo Merge with Chris's
  // ****************************************************************************
  /**
   * Checks that the passed String is a number eg. 23.45
   * 
   * @return true means the passed string is all numbers.
   */
  public static boolean isNumeric(String str) {
    try {
      Double.parseDouble(str);
      return true;
    } catch (NumberFormatException ex) {
      return false;
    }
  }

  public String getMonthDescription(Date date) {
    DateFormat df = DateFormat.getDateInstance();
    L.debug("" + df.format(date));
    return null;

  }

  /**
   * Determines the difference in months between two dates. Gopal - todo - look at it a bit more closely - Currently
   * does a very rough rounding for fractions of a month on each side. Is this necessary?
   */
  public static int getMonthDifference(Date early, Date late) {
    int monthDiff = 0;
    int sign = 1;
    Date date1 = Util.getDateWithNoTime(early);
    Date date2 = Util.getDateWithNoTime(late);
    if (date1.after(date2)) {
      sign = -1;
      // Swop the dates
      Date tempDate = date1;
      date1 = date2;
      date2 = tempDate;
    }
    Calendar cal1 = new GregorianCalendar();
    cal1.setTime(date1);
    Calendar cal2 = new GregorianCalendar();
    cal2.setTime(date2);
    int dayOfMonth1 = cal1.get(Calendar.DAY_OF_MONTH);
    int dayOfMonth2 = cal2.get(Calendar.DAY_OF_MONTH);
    int month1 = cal1.get(Calendar.MONTH);
    int month2 = cal2.get(Calendar.MONTH);
    int year1 = cal1.get(Calendar.YEAR);
    int year2 = cal2.get(Calendar.YEAR);
    if (year1 == year2) {
      monthDiff = month2 - month1;
    } else {
      monthDiff = (11 - month1) + (month2 + 1) + (year2 - year1 - 1) * 12;
    }
    // Rough rounding - assume that more that 20 days warrants an extra month
    if ((dayOfMonth2 - dayOfMonth1) > 20) {
      monthDiff++;
    }
    return monthDiff * sign;
  }

  /**
   * Returns true if s1 and s2 are equal. Two strings are considered equal if: <br>
   * 1. They are both null or empty strings. Note that if s1 = null and s2 ="", this is considered equal. <br>
   * 2. They have the same characters. <br>
   * Note, leading and trailing spaces are ignored in comparisons, as well as case.
   */
  public static boolean areStringsEqual(String s1, String s2) {
    if (Util.isEmpty(s1) && Util.isEmpty(s2)) {
      return true;
    }

    if (Util.isEmpty(s1) && !Util.isEmpty(s2)) {
      return false;
    }

    if (!Util.isEmpty(s1) && Util.isEmpty(s2)) {
      return false;
    }
    // String s2a = s2.trim();
    if (s1.trim().equalsIgnoreCase(s2.trim())) {
      return true;
    } else {
      return false;
    }
  }

  /**
   * Takes the given string and simply returns a new string where the first char is lowercase. This is in accordance
   * with jave names for variables.
   */
  public static String getJavaVariableName(String name) {
    return name.substring(0, 1).toLowerCase() + name.substring(1);
  }

  /**
   * This method will, along with the helper method replaceEscapeSequences(), search a strring for all occurrences of
   * defined escape sequences and replace them with the corresponding strings.
   * 
   * @param toSearch
   * @return String
   */
  public static String replaceEscapeSequences(String toSearch) {
    for (int i = 0; i < ESCAPE_SEQUENCES.length; i++)
      toSearch = replaceEscapeSequence(toSearch, ESCAPE_SEQUENCES[i][0], ESCAPE_SEQUENCES[i][1]);
    return toSearch;
  }

  /**
   * @see replaceEscapeSequences(String)
   * @param toSearch
   * @param toSearchFor
   * @param toReplaceWith
   * @return String
   */
  private static String replaceEscapeSequence(String toSearch, String toSearchFor, String toReplaceWith) {
    return replaceString(toSearch, toSearchFor, toReplaceWith);
  }

  /**
   * Takes the given string toSearchFor, and replaces all occurrences of string toSearchfor with string toReplaceWith.
   * 
   * @return String
   */
  public static String replaceString(String toSearch, String toSearchFor, String toReplaceWith) {
    StringBuffer buffer = null;
    int pos = -1;
    while ((pos = toSearch.indexOf(toSearchFor)) != -1) {
      buffer = new StringBuffer(toSearch);
      buffer.replace(pos, pos + toSearchFor.length(), toReplaceWith);
      toSearch = buffer.toString();
    }
    return (buffer == null) ? toSearch : buffer.toString();
  }

  /**
   * Expects to search a String formatted as xml for the content of an element.
   * 
   * @param xmlElement
   * @param xmlString
   * @return content as String
   */
  public static String extractXMLElementContent(String xmlElement, String xmlString) {
    if (Util.isEmpty(xmlElement)) {
      // L.debug("xmlElement is empty");
      return null;
    }
    String startElement = "<" + xmlElement + ">";
    String endElement = "</" + xmlElement + ">";
    // L.debug("startElement="+startElement);
    // L.debug("xmlString="+xmlString);
    int index = xmlString.indexOf(startElement);
    if (index != -1) {
      int elementOffset = index + startElement.length();
      int endElementOffset = xmlString.indexOf(endElement);
      if (endElementOffset == -1) {
        // L.debug("No end element found: "+xmlElement);
        return null;
      }
      String content = xmlString.substring(elementOffset, endElementOffset);
      // L.debug("content of "+xmlElement+"="+content);
      return content.trim();
    } else {
      // L.debug("No such element found: "+xmlElement);
      return null;
    }
  }

  /**
   * Expects to search a String formatted as xml for the content of an element.
   * 
   * @param xmlElement
   * @param xmlString
   * @param remainder
   *          The remainder of the string after this element. It is assumed that remainder is empty.
   * @return content as String
   */
  public static String extractXMLElementContent(String xmlElement, String xmlString, StringBuffer remainder) {
    if (Util.isEmpty(xmlElement)) {
      // L.debug("xmlElement is empty");
      return null;
    }
    String startElement = "<" + xmlElement + ">";
    String endElement = "</" + xmlElement + ">";
    int index = xmlString.indexOf(startElement);
    if (index != -1) {
      int elementOffset = index + startElement.length();
      int endElementOffset = xmlString.indexOf(endElement);
      if (endElementOffset == -1) {
        // L.debug("No end element found: "+xmlElement);
        return null;
      }
      String content = xmlString.substring(elementOffset, endElementOffset);
      // L.debug("content of "+xmlElement+"="+content);
      remainder.append(xmlString.substring(endElementOffset + endElement.length()));
      return content.trim();
    } else {
      // L.debug("No such element found: "+xmlElement);
      return null;
    }
  }

  /**
   * Place the contents of a String in a fixed size byte array and pad array with spaces
   * 
   * @param value
   * @param fixedLength
   * @param filler
   * @return fixed size byte array padded with specified byte
   */
  public static byte[] getFixedLengthByteArray(String value, int fixedLength, byte fill) throws IOException {
    byte[] bytes = null;
    if (value == null) {
      bytes = new byte[fixedLength];
      Arrays.fill(bytes, (byte) fill);
    } else {
      ByteArrayOutputStream out = new ByteArrayOutputStream(fixedLength);
      out.write(value.getBytes());
      if (out.size() < fixedLength) {
        byte[] filler = new byte[fixedLength - out.size()];
        Arrays.fill(filler, (byte) fill);
        out.write(filler);
      }
      out.close();
      bytes = out.toByteArray();
    }
    return bytes;
  }

  /**
   * Place the contents of a String in a fixed size byte array and pad array with specified byte
   * 
   * @param value
   * @param fixedLength
   * @param filler
   * @return fixed size byte array padded with spaces
   */
  public static byte[] getFixedLengthByteArrayRightAligned(String value, int fixedLength, byte fill) throws IOException {
    byte[] bytes = null;
    if (value == null) {
      bytes = new byte[fixedLength];
      Arrays.fill(bytes, (byte) fill);
    } else {
      ByteArrayOutputStream out = new ByteArrayOutputStream(fixedLength);
      if (value.length() < fixedLength) {
        byte[] filler = new byte[fixedLength - value.length()];
        Arrays.fill(filler, (byte) fill);
        out.write(filler);
      }
      out.write(value.getBytes());
      out.close();
      bytes = out.toByteArray();
    }
    return bytes;
  }

  /**
   * Formats a long string into multiple xml CDATA segments no more than width wide
   * 
   * @param in
   * @param totalWidth
   * @return Formatted segments
   */
  public static String getFormattedAsFixedWidthCDATA(String in, int totalWidth) {
    StringBuffer out = new StringBuffer();
    String pre = "<![CDATA[";
    String post = "]]>";
    int width = totalWidth - (pre.length() + post.length());
    int lines = in.length() / width;
    int rest = in.length() % width;
    for (int i = 0; i < lines; i++) {
      out.append(pre);
      int offset = i * width;
      out.append(in.substring(offset, offset + width));
      out.append(post);
    }
    if (rest > 0) {
      out.append(pre);
      int offset = lines * width;
      out.append(in.substring(offset, offset + rest));
      out.append(post);
    }
    return out.toString();
  }

  /**
   * Formats a long string into multiple xml CDATA segments no more than width wide
   * 
   * @param in
   * @param totalWidth
   * @return Formatted segments
   */
  public static String getFormattedAsCDATA(String in) {
    String pre = "<![CDATA[";
    String post = "]]>";
    return pre + in + post;
  }

  /**
   * This takes an object instance and populate it using reflection from the xml string
   * 
   * @param to
   * @param from
   * @return Object populated from xml
   */
  public static Object xmlToObject(Object to, String from) throws IntrospectionException, IllegalAccessException,
      InvocationTargetException {
    if (Util.isEmpty(from))
      return null;
    BeanInfo bi = Introspector.getBeanInfo(to.getClass());
    PropertyDescriptor pd[] = bi.getPropertyDescriptors();
    boolean noChanges = true;
    for (int i = 0; i < pd.length; i++) {
      String xmlElementName = propertyNameToXMLElementName(pd[i].getName());
      if (xmlElementName != null) {
        String content = extractXMLElementContent(xmlElementName, from);
        Method writeMethod = pd[i].getWriteMethod();
        if (content != null && writeMethod != null) {
          writeMethod.invoke(to, new Object[] { content });
          noChanges = false;
        }
        // if (writeMethod == null)
        // L.debug("Setter method for "+pd[i].getName()+
        // " is null. Maybe it was declared private?");
      }
    }
    if (noChanges)
      return null;
    else
      return to;
  }

  /**
   * @param string
   */
  private static String propertyNameToXMLElementName(String propName) {
    StringBuffer xmlElementName = new StringBuffer();
    xmlElementName.append(Character.toUpperCase(propName.charAt(0)));
    xmlElementName.append(propName.substring(1));
    return xmlElementName.toString();
  }

  public static void main(String[] args) throws Exception {

  }

  /**
   * This method uses reflection to stringify an array of objects
   * 
   * @param array
   * @return Stringified version of array
   * @throws IllegalArgumentException
   * @throws IntrospectionException
   * @throws IllegalAccessException
   * @throws InvocationTargetException
   */
  private static String arrayToString(Object[] array) throws IllegalArgumentException, IntrospectionException,
      IllegalAccessException, InvocationTargetException {
    StringWriter out = new StringWriter();
    for (int i = 0; i < array.length; i++) {
      out.write("[" + i + "]=" + classToString(array[i]));
      out.write("\n");
    }
    return out.toString();
  }

  /**
   * This method uses reflection to stringify an array of objects
   * 
   * @param col
   * @return Stringified version of collection
   * @throws IllegalArgumentException
   * @throws IntrospectionException
   * @throws IllegalAccessException
   * @throws InvocationTargetException
   */
  private static String collectionToString(Collection col) throws IllegalArgumentException, IntrospectionException,
      IllegalAccessException, InvocationTargetException {
    StringWriter out = new StringWriter();
    int i = 0;
    for (Iterator iter = col.iterator(); iter.hasNext();) {
      out.write("[" + i + "]=" + classToString(iter.next()));
      out.write("\n");
    }
    return out.toString();
  }

  /**
   * This method uses reflection to stringify an object
   * 
   * @param obj
   * @return Stringified version of object
   */
  public static String classToString(Object obj) throws IntrospectionException, IllegalArgumentException,
      IllegalAccessException, InvocationTargetException {
    if (obj == null)
      return "obj is null";
    StringWriter out = new StringWriter();
    BeanInfo bi = Introspector.getBeanInfo(obj.getClass());
    PropertyDescriptor[] pds = bi.getPropertyDescriptors();
    Method readMethod = null;
    for (int i = 0; i < pds.length; i++) {
      readMethod = pds[i].getReadMethod();
      if (readMethod != null) {
        L.debug("ReadMethod = " + readMethod.getName() + " obj = " + obj.getClass());
        Object value = readMethod.invoke(obj);
        if (value != null) {
          if (value.getClass().isPrimitive() || value instanceof String || value instanceof Date
              || value instanceof Integer) {
            out.write(pds[i].getName());
            out.write("=");
            out.write(value.toString());
            out.write("\n");
          } else if (pds[i].getName().equals("interface") || pds[i].getName().equals("class")) {
          } else if (pds[i].getPropertyType().isArray()) {
            L.debug("Name = " + pds[i].getName() + " and Property type = " + pds[i].getPropertyType());
            out.write(arrayToString((Object[]) value));
          } else if (value instanceof Collection) {
            out.write(collectionToString((Collection) value));
          } else {
            out.write(pds[i].getName());
            out.write("=[\n");
            out.write(classToString(value));
            out.write("]\n");
          }
        } else {
          out.write(pds[i].getName());
          out.write("=null\n");
        }
      }
    }
    return out.toString();
  }

  /**
   * Total over an array of integers
   * 
   * @param updateCounts
   * @return
   */
  public static int total(int[] ints) {
    int total = 0;
    for (int i = 0; i < ints.length; i++) {
      total += ints[i];
    }
    return total;
  }

  /**
   * @param date
   * @return
   */
  public static Calendar getCalendar(Date date) {
    Calendar cal = Calendar.getInstance();
    cal.setTime(date);
    return cal;
  }

  /**
   * @return
   */
  public static Date getTodaysDate() {
    Date todayWithTime = new Date();
    return getDateWithNoTime(todayWithTime);
  }

  /**
   * This method will return a file instance if the file exists, otherwise null
   * 
   * @param dir
   * @param fileName
   * @return File
   */
  public static File openInputStream(String dir, String fileName) {
    String fullFileName = getFileName(dir, fileName);
    File f = new File(fullFileName);
    if (!f.exists())
      return null;
    else
      return f;
  }

  /**
   * Returns a date given a month-year value - either as first or last day of the month.
   * 
   * @param monthYear
   * @param endOfMonth
   * @return
   */
  public static java.sql.Date getDateFromMonthYear(int monthYear, boolean endOfMonth) {
    try {
      String monthYearString = new Integer(monthYear).toString();
      if (monthYearString.length() != 6)
        throw new DomainRuntimeException("monthYear should be in format yyyymm. Instead it looks like this: "
            + monthYear);
      Date monthYearDate = rawDateFormat.parse(new Integer(monthYear).toString() + "01");
      if (endOfMonth) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(monthYearDate);
        cal.add(Calendar.MONTH, 1);
        cal.add(Calendar.DAY_OF_YEAR, -1);
        return new java.sql.Date(cal.getTime().getTime());
      } else
        return new java.sql.Date(monthYearDate.getTime());
    } catch (ParseException e) {
      L.debug("error", e);
      throw new DomainRuntimeException(e);
    }
  }

  /**
   * @param endMonthYear
   * @param i
   * @return
   */
  public static Integer incrementMonthYear(Integer endMonthYear, int i) {
    Calendar date = Calendar.getInstance();
    date.setTime(getDateFromMonthYear(endMonthYear.intValue(), true));
    date.add(Calendar.MONTH, i);
    return getMonthYearFromDate(date);
  }

  /**
   * Takes a string containing words delimited by space, extracts the first character of each and builds an uppercased
   * string of the characters delimited by space. Typically used to build initials from a list of names
   * 
   * @param words
   */
  public static String getFirstCharOfEveryWordAsString(String words) {
    StringTokenizer tkn = new StringTokenizer(words, " ");
    String result = "";
    boolean firstPass = true;
    while (tkn.hasMoreTokens()) {
      if (firstPass)
        firstPass = false;
      else
        result += " ";
      result += tkn.nextToken().charAt(0);
    }
    return result.toUpperCase();
  }

  /**
   * @return
   */
  private static Integer getMonthYearFromDate(Calendar date) {
    StringBuffer monthYear = new StringBuffer(6);
    monthYear.append(String.valueOf(date.get(Calendar.YEAR)));
    int month = date.get(Calendar.MONTH);
    monthYear.append(month < 10 ? "0" : "");
    monthYear.append(month);
    return new Integer(monthYear.toString());
  }

  /**
   * This method retrieves the first word from the string delimited by space.
   * 
   * @param string
   * @return
   */
  public static String getFirstWord(String string) {
    if (isEmpty(string))
      return EMPTY_STRING;
    int spaceIndex = string.indexOf(' ');
    if (spaceIndex == -1)
      return string;
    return string.substring(0, spaceIndex);
  }

  /**
   * Method to help calculate a checksum for the given string.
   */
  public static long calculateChecksum(String data) {
    byte[] bytes = data.getBytes();
    return calculateChecksum(bytes);
  }

  /**
   * Method to help calculate a checksum for the given byte array.
   */
  public static long calculateChecksum(byte[] bytes) {
    Checksum checksumEngine = new Adler32();
    checksumEngine.update(bytes, 0, bytes.length);
    return checksumEngine.getValue();
  }

  /**
   * Appends two byte arrays and returns the result.
   */
  public static byte[] appendByteArrays(byte[] a, byte[] b) {
    byte[] app = new byte[a.length + b.length];
    System.arraycopy(a, 0, app, 0, a.length);
    System.arraycopy(b, 0, app, a.length, b.length);
    return app;
  }

  /**
   * Zips (compresses) a file
   * 
   * @param sourceFile
   * @param zipFilename
   */
  public static void zipFile(String sourceFile, String zipFilename) {
    ZipOutputStream out = null;
    BufferedInputStream origin = null;
    try {
      FileOutputStream dest = new FileOutputStream(zipFilename);
      CheckedOutputStream checksum = new CheckedOutputStream(dest, new Adler32());
      out = new ZipOutputStream(new BufferedOutputStream(checksum));
      out.setMethod(ZipOutputStream.DEFLATED);
      byte data[] = new byte[ZIP_BUFFER_SIZE];
      FileInputStream fi = new FileInputStream(sourceFile);
      origin = new BufferedInputStream(fi, ZIP_BUFFER_SIZE);
      ZipEntry zipEntry = new ZipEntry(sourceFile);
      out.putNextEntry(zipEntry);
      int count;
      while ((count = origin.read(data, 0, ZIP_BUFFER_SIZE)) != -1) {
        out.write(data, 0, count);
      }
      origin.close();
      out.close();
    } catch (IOException ioe) {
      throw new DomainRuntimeException("Unable to zip file " + sourceFile, ioe);
    } finally {
      try {
        if (origin != null)
          origin.close();
        if (out != null)
          out.close();
      } catch (IOException e) {
      }
    }

  }

  /**
   * Unzips (decompresses) a file
   * 
   * @param zipFile
   * @param destinationFile
   */
  public static void unzipFile(String zipFile, String destinationFile) {
    BufferedOutputStream dest = null;
    try {
      FileInputStream fis = new FileInputStream(zipFile);
      CheckedInputStream checksum = new CheckedInputStream(fis, new Adler32());
      ZipInputStream zis = new ZipInputStream(new BufferedInputStream(checksum));
      ZipEntry entry;
      while ((entry = zis.getNextEntry()) != null) {
        int count;
        byte data[] = new byte[ZIP_BUFFER_SIZE];
        // write the files to the disk

        FileOutputStream fos = new FileOutputStream(destinationFile + getFileShortName(entry.getName()));
        dest = new BufferedOutputStream(fos, ZIP_BUFFER_SIZE);
        while ((count = zis.read(data, 0, ZIP_BUFFER_SIZE)) != -1) {
          dest.write(data, 0, count);
        }
        dest.flush();
        dest.close();
      }
      zis.close();
    } catch (IOException ioe) {
      throw new DomainRuntimeException("Unable to unzip file " + zipFile, ioe);
    } finally {
      try {
        if (dest != null)
          dest.close();
      } catch (IOException e) {
      }
    }
  }

  public static byte[] compressBytes(byte[] inputBytes) {
    Deflater compressor = new Deflater();
    compressor.setLevel(Deflater.BEST_COMPRESSION);

    // Give the compressor the data to compress
    compressor.setInput(inputBytes);
    compressor.finish();

    // Create an expandable byte array to hold the compressed data.
    // You cannot use an array that's the same size as the orginal because
    // there is no guarantee that the compressed data will be smaller than
    // the uncompressed data.
    ByteArrayOutputStream bos = new ByteArrayOutputStream(inputBytes.length);

    // Compress the data
    byte[] buf = new byte[1024];
    while (!compressor.finished()) {
      int count = compressor.deflate(buf);
      bos.write(buf, 0, count);
    }
    try {
      bos.close();
    } catch (IOException e) {
    }

    // Get the compressed data
    return bos.toByteArray();
  }
  /**
   * Takes a date and returns a string formated. Converse of getDate().
   * 
   * @param date
   * @return String
   */
  public static String getDateTimeStringFromDate(java.util.Date date) {
    return dateTimeFormat.format(date);
  }
}
