package za.co.momentum.recurring.nurse.client.sao.appreq;

import java.util.Calendar;
import java.util.Collection;

import javax.xml.rpc.ServiceException;

import org.apache.log4j.Logger;

import za.co.momentum.recurring.nurse.client.model.ClientSecurityManager;
import za.co.momentum.recurring.nurse.client.model.ErrorMessages;
import za.co.momentum.recurring.nurse.client.sao.AbstractServiceManager;
import za.co.momentum.recurring.nurse.client.sao.ServiceAction;
import za.co.momentum.recurring.nurse.client.tools.SendErrorEmail;
import za.co.momentum.recurring.nurse.util.DBHelper;
import za.co.momentum.recurring.nurse.util.TNLogger;
import za.co.momentum.risk.fastlane.exceptions.EntityNotFoundException;
import za.co.momentum.risk.fastlane.nurse.dto.AppDetailTO;
import za.co.momentum.risk.fastlane.nurse.dto.OriginatorDetailsTO;
import za.co.momentum.risk.fastlane.nurse.dto.PdfFormTO;
import za.co.momentum.risk.fastlane.nurse.dto.RoleTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;
import za.co.momentum.risk.fastlane.nurse.exceptions.AppReqLockedException;
import za.co.momentum.risk.fastlane.nurse.exceptions.TimeSlotNotAvailableException;
import za.co.momentum.risk.fastlane.nurse.services.appreq.AppReqService;
import za.co.momentum.risk.fastlane.nurse.services.appreq.AppReqServiceRest;

/**
 * The AppReqServiceManager class is used to access the AppReqService. We need
 * to use this class because it will alow us the ability to make use of a
 * timeout mechanism.
 * 
 * Look at the system.properties file under base. The property
 * "timeout.client=10" will define the timeout of a service call from the client
 * in seconds.
 * 
 * This class is a synchronized singleton and must be used in that way. Call the
 * static method getInstance() to receive an instance of AppReqServiceManager.
 * 
 * @author NideBruyn Created on 10 Nov, 2005 Risk & Savings
 * 
 */
public class AppReqServiceManager extends AbstractServiceManager {
	/*
	 * A single instance of this class. It uses the singleton pattern.
	 */
	private static AppReqServiceManager instance;

	/*
	 * This is the LOG instance.
	 */
	private static final Logger LOG = Logger
			.getLogger(AppReqServiceManager.class);

	/**
	 * This is the hidden constructor so that we can make the class a
	 * singleton...
	 * 
	 */
	protected AppReqServiceManager() {
	}

	/**
	 * Method for returning an instance of this class.
	 * 
	 * @return
	 */
	public static AppReqServiceManager getInstance() {
		if (instance == null) {
			synchronized (AppReqServiceManager.class) {
				if (instance == null) {
					instance = new AppReqServiceManager();
				}
			}
		}
		return instance;
	}

	/**
	 * This is a helper method for returning the AppReqService.
	 * 
	 * @return
	 * @throws ServiceException
	 */
	private AppReqService getAppReqService() throws ServiceException {
		// AppReqServiceServiceLocator a = new AppReqServiceServiceLocator();
		// // a.setCacheWSDL(false);
		// return a.getAppReqService();

		return AppReqServiceRest.getInstance();

	}

	/**
	 * This method will return all appointment requests.
	 * 
	 * @return
	 */
	public AppDetailTO[] getToBeScheduledAppReqs() {
		LOG.debug("entering getToBeScheduledAppReqs");
		final String name = ClientSecurityManager.getInstance().getUser()
				.getUsername();
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().getToBeScheduledAppReqs(name);
			}
		});
		LOG.info("Finished calling tobeShceduled app reqs and it is "
				+ returnObj);
		if (returnObj instanceof AppDetailTO[]) {
			return (AppDetailTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getToBeScheduledAppReqs", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * Retrieves duplicate client on same policy number
	 * 
	 * @param
	 */
	public AppDetailTO[] getClientsWithSamePolicyNumber(final AppDetailTO appreq) {
		LOG.debug("entering getClientsWithSamePolicyNumber");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().findClientsWithSamePolicyNum(
						appreq.getId());
			}
		});
		if (returnObj instanceof AppDetailTO[]) {
			return (AppDetailTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getClientsWithSamePolicyNumber", "AppReqServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This method will return the completed appointment requests.
	 * 
	 * @return
	 */
	public AppDetailTO[] getCompletedAppReqs() {
		LOG.debug("entering getCompletedAppReqs");
		final String name = ClientSecurityManager.getInstance().getUser()
				.getUsername();
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().getCompletedAppReqs(name);
			}
		});
		if (returnObj instanceof AppDetailTO[]) {
			return (AppDetailTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getCompletedAppReqs", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * Get the pending appoingtment requests.
	 * 
	 * @return
	 */
	public AppDetailTO[] getPendingAppReqs() {
		LOG.debug("entering getPendingAppReqs");
		final String name = ClientSecurityManager.getInstance().getUser()
				.getUsername();
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().getPendingAppReqs(name);
			}
		});
		if (returnObj instanceof AppDetailTO[]) {
			return (AppDetailTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getPendingAppReqs", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * Get the history detail appointment requests.
	 * 
	 * @return
	 */
	public OriginatorDetailsTO getAppreqByID(final long appReqID) {
		LOG.debug("entering getAppreqByID");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService()
						.getAppReqById(
								ClientSecurityManager.getInstance().getUser(),
								appReqID);
			}
		});
		if (returnObj instanceof OriginatorDetailsTO) {
			return (OriginatorDetailsTO) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getAppreqByID", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * Get the appointment details.
	 * 
	 * @param policyNumber
	 * @param clientNumber
	 * @return
	 */
	public OriginatorDetailsTO getAppReqDetail(final long id) {
		LOG.info("entering getAppReqDetail");
		final String uname = ClientSecurityManager.getInstance().getUser()
				.getUsername();
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().getAppReqById(
						ClientSecurityManager.getInstance().getUser(), id);
			}
		});
		if (returnObj instanceof OriginatorDetailsTO) {
			return (OriginatorDetailsTO) returnObj;
		} else if (returnObj instanceof AppReqLockedException) {
			ErrorMessages.showMessage(ErrorMessages.M10,
					new String[] { "AppReqLockedException" });
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getAppReqDetail", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * Get the appointment details new.
	 * 
	 * @param policyNumber
	 * @param clientNumber
	 * @return
	 */
	/*
	 * public AppReqDetailsTONEW getAppReqDetailNew(final Long appreqId) {
	 * LOG.info("entering getAppReqDetail"); final String uname =
	 * ClientSecurityManager.getInstance().getUser() .getUsername(); Object
	 * returnObj = executeServiceAction(new ServiceAction( new ServiceListener()
	 * { public Object performService() throws Exception { return
	 * getAppReqService().getAppReqNEW(uname, appreqId); } })); if (returnObj
	 * instanceof AppReqDetailsTONEW) { return (AppReqDetailsTONEW) returnObj; }
	 * else if (returnObj instanceof AppReqLockedException) {
	 * LOG.info("Exception message=" + ((AppReqLockedException)
	 * returnObj).getErrorMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M10, new String[] {
	 * ((AppReqLockedException) returnObj).getErrorMessage(), "" }); return
	 * null; } else if (returnObj instanceof Exception) { LOG.error("ERR: (" +
	 * returnObj + ") " + ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M0); return null; } return null;
	 * }
	 */
	/**
	 * This method will set the client declined action
	 * 
	 * @param dc
	 * @param appreqDetail
	 * @return
	 */
	public AppDetailTO setClientDeclinedAction(final UserTO user,
			final UserTO admin, final AppDetailTO appreqDetail) {
		LOG.debug("entering setClientDeclinedAction");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().setClientDeclinedAction(user, admin,
						appreqDetail);
			}
		});
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"setClientDeclinedAction", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will save the appReq
	 * 
	 * @param appReqTO
	 * @return AppReqDetailsTO
	 */
	public AppDetailTO saveAppReq(final UserTO nurse, final UserTO admin,
			final AppDetailTO appReq) {
		LOG.debug("entering saveAppReq");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().saveAppReq(nurse, admin, appReq);
			}
		});
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj, "saveAppReq",
					"AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will set the nurse cancelled action
	 * 
	 * @param nurse
	 * @param appReqTO
	 * @return AppReqDetailsTO
	 */
	public AppDetailTO setNurseCancelledAction(final UserTO nurse,
			final UserTO admin, final AppDetailTO appReqTO) {
		LOG.debug("entering setNurseCancelledAction");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().setNurseCancelledAction(nurse, admin,
						appReqTO);
			}
		});
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"setNurseCancelledAction", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will set the client cancelled action
	 * 
	 * @param nurse
	 * @param appReqTO
	 * @return AppReqDetailsTO
	 */
	public AppDetailTO setClientCancelledAction(final UserTO nurse,
			final UserTO admin, final AppDetailTO appReqTO) {
		LOG.debug("entering setClientCancelledAction");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().setClientCancelledAction(nurse,
						admin, appReqTO);
			}
		});
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"setClientCancelledAction", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will set the appointment done action
	 * 
	 * @param nurse
	 * @param appReqTO
	 * @return AppReqDetailsTO
	 */
	public String[] setAppointmentDoneAction(final UserTO nurse,
			final UserTO admin, final AppDetailTO appReqTO,
			final PdfFormTO[] pdfFormTO) throws Exception {
		LOG.debug("entering setAppointmentDoneAction");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().setAppointmentDoneAction(nurse,
						admin, appReqTO, pdfFormTO);
			}
		});
		if (returnObj instanceof Exception)
			throw (Exception) returnObj;
		return (String[]) returnObj; // This must throw
		// ClassCastException if
		// a strange object is returned... bug.
	}

	/**
	 * This method will set get all the nurseTO's for a specific suburb
	 * 
	 * @param suburbId
	 * @return NurseTO[]
	 */
	public RoleTO[] getNursePerSuburb(final String suburbName) {
		LOG.debug("entering getNursePerSuburb");

		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().getNursesBySuburb(suburbName);
			}
		});
		if (returnObj instanceof RoleTO[]) {
			return (RoleTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getNursePerSuburb", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will set get all the appreqs for a specific date and region
	 * 
	 * @param nurse
	 * @param date
	 * @return AppReqDetailsTO[]
	 */
	public AppDetailTO[] getAppreqsByDate(final RoleTO nurse,
			final Calendar theDay) {
		LOG.debug("entering getAppreqsByDate");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				System.out.println("Nurse: " + nurse.getId() + " & Date: "
						+ theDay.getTimeInMillis());
				return getAppReqService().getAppReqsByNurseAndDate(
						nurse.getId().longValue(), theDay.getTimeInMillis());
			}
		});
		if (returnObj instanceof AppDetailTO[]) {
			return (AppDetailTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			// Now try and get the details from local database
			TNLogger.debug("Calling local db to get apps");
			AppDetailTO[] apps = DBHelper.getAllAppReqs(ClientSecurityManager
					.getInstance().getUser().getUsername(),
					theDay.get(Calendar.DAY_OF_YEAR));
			if (apps == null) {
				LOG.error("ERR: (" + returnObj + ") "
						+ ((Exception) returnObj).getMessage());
				SendErrorEmail.sendErrorMail((Exception) returnObj,
						"getAppreqsByDate", "AppReqServiceManager");
				ErrorMessages.showMessage(ErrorMessages.M0);
				return null;
			}
			return apps;
		}
		return null;
	}

	/**
	 * This method will make a appointment status booked
	 * 
	 * @param AppreqDetail
	 * @return
	 */
	public AppDetailTO bookAppointment(final UserTO dc, final UserTO admin,
			AppDetailTO detail) {
		LOG.debug("entering bookAppointment");
		final AppDetailTO appreqDetail = detail;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().bookAppointment(dc, admin,
						appreqDetail);
			}
		});
		LOG.info("bookAppointment returnobj=" + returnObj);
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof TimeSlotNotAvailableException) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M19);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"bookAppointment", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will make a personal appointment status
	 * 
	 * @param AppreqDetail
	 * @return
	 */
	public AppDetailTO bookPersonalAppointment(final UserTO nurse,
			AppDetailTO detail) {
		LOG.debug("entering bookPersonalAppointment");
		final AppDetailTO appreqDetail = detail;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().bookPersonalAppointment(nurse,
						appreqDetail);
			}
		});
		LOG.info("bookPersonalAppointment returnobj=" + returnObj);
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof TimeSlotNotAvailableException) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M19);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"bookPersonalAppointment", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will update a personal appointment
	 * 
	 * @param AppreqDetail
	 * @return
	 */
	public AppDetailTO updatePersonalAppointment(final UserTO nurse,
			AppDetailTO detail) {
		LOG.debug("entering updatePersonalAppointment");
		final AppDetailTO appreqDetail = detail;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().updatePersonalAppointment(nurse,
						appreqDetail);
			}
		});
		LOG.info("updatePersonalAppointment returnobj=" + returnObj);
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof TimeSlotNotAvailableException) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M19);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"updatePersonalAppointment", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will update a personal appointment
	 * 
	 * @param AppreqDetail
	 * @return
	 */
	public boolean deletePersonalAppointment(final UserTO nurse,
			AppDetailTO detail) {
		LOG.debug("entering deletePersonalAppointment");
		final AppDetailTO appreqDetail = detail;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				getAppReqService().deletePersonalAppointment(nurse,
						appreqDetail);
				return new Boolean(true);
			}
		});
		LOG.info("deletePersonalAppointment returnobj=" + returnObj);
		if (returnObj instanceof Boolean) {
			return ((Boolean) returnObj).booleanValue();
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"deletePersonalAppointment", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return false;
		}
		return false;
	}

	/**
	 * This method will make a appointment status closed
	 * 
	 * @param AppreqDetail
	 * @return
	 */
	public AppDetailTO closeAppointment(final UserTO dc, AppDetailTO detail) {
		LOG.debug("entering closeAppointment");
		final AppDetailTO appreqDetail = detail;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				getAppReqService().closeAppReq(dc, appreqDetail);
				return null;
			}
		});
		if (returnObj == null) {
			return detail;
		} else if (returnObj instanceof EntityNotFoundException) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"closeAppointment", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	/**
	 * This method will return the requirements for a appreq
	 * 
	 * @param AppreqDetail
	 * @return AppReqRequirementTO[]
	 */
	// public AppReqRequirementTO[] getRequirements(AppReqDetailsTO detail) {
	// LOG.debug("entering AppReqRequirementTO");
	// final AppReqDetailsTO appreqDetail = detail;
	// Object returnObj = executeServiceAction(new ServiceAction(
	// new ServiceListener() {
	// public Object performService() throws Exception {
	// return getAppReqService().getLatestRequirements(appreqDetail);
	// }
	// }));
	// if (returnObj instanceof AppReqRequirementTO[]) {
	// return (AppReqRequirementTO[]) returnObj;
	// } else if (returnObj instanceof Exception) {
	// LOG.error("ERR: (" + returnObj + ") "
	// + ((Exception) returnObj).getMessage());
	// SendErrorEmail.sendErrorMail((Exception) returnObj, "getRequirements",
	// "AppReqServiceManager");
	// ErrorMessages.showMessage(ErrorMessages.M0);
	// return null;
	// }
	// return null;
	// }

	/**
	 * This method will make a appointment status booked
	 * 
	 * @param AppreqDetail
	 * @return
	 */
	public AppDetailTO unscheduleApp(final UserTO dc, final UserTO admin,
			AppDetailTO detail) {
		LOG.debug("entering unscheduleApp");
		final AppDetailTO appreqDetail = detail;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getAppReqService().setUnScheduleAction(dc, admin,
						appreqDetail);
			}
		});
		LOG.info("unscheduleApp returnobj=" + returnObj);
		if (returnObj instanceof AppDetailTO) {
			return (AppDetailTO) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"unscheduleApp", "AppReqServiceManager");
			ErrorMessages.showMessage(ErrorMessages.M0);
			return null;
		}
		return null;
	}

	public void sendGenericEmail(final String fromAddress,
			final String toAddress, final String subject,
			final String bodyText, final Collection<String> fileNames)
			throws Exception {

		LOG.debug("Sending email to.. " + toAddress + "... From..."
				+ fromAddress);

		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				getAppReqService().sendGenericEmail(fromAddress, toAddress,
						subject, bodyText, null);
				return null;
			}
		});

		if (returnObj == null) {
		} else if (returnObj instanceof EntityNotFoundException) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			throw new Exception();
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + returnObj + ") "
					+ ((Exception) returnObj).getMessage());
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"noMedPersonAvailable", "AppReqServiceManager");
			throw new Exception();
		}
	}

}
