package za.co.momentum.recurring.nurse.client.sao.user;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.rpc.ServiceException;

import org.apache.log4j.Logger;

import za.co.momentum.recurring.nurse.client.model.ErrorMessages;
import za.co.momentum.recurring.nurse.client.sao.AbstractServiceManager;
import za.co.momentum.recurring.nurse.client.sao.ServiceAction;
import za.co.momentum.recurring.nurse.client.tools.SendErrorEmail;
import za.co.momentum.risk.fastlane.exceptions.DuplicateEntityException;
import za.co.momentum.risk.fastlane.exceptions.EntityNotFoundException;
import za.co.momentum.risk.fastlane.exceptions.UnauthorizedUserException;
import za.co.momentum.risk.fastlane.nurse.dto.RoleTO;
import za.co.momentum.risk.fastlane.nurse.dto.SuburbTO;
import za.co.momentum.risk.fastlane.nurse.dto.UserTO;
import za.co.momentum.risk.fastlane.nurse.services.user.UserService;
import za.co.momentum.risk.fastlane.nurse.services.user.UserServiceRest;

/**
 * The UserServiceManager class is used to access the UserService's. We need to
 * use this class because it will alow us the ability to make use of a timeout
 * mechanism.
 * 
 * Look at the system.properties file under base. The property
 * "timeout.client=10" will define the timeout of a service call from the client
 * in seconds.
 * 
 * This class is a synchronized singleton and must be used in that way. Call the
 * static method getInstance() to receive an instance of UserServiceManager.
 * 
 * @author NideBruyn Created on 9 Nov, 2005 Risk & Savings
 * 
 */
public class UserServiceManager extends AbstractServiceManager {
	/*
	 * A single instance of this class. It uses the singleton pattern.
	 */
	private static UserServiceManager instance;
	/*
	 * This is the LOG instance.
	 */
	private static final Logger LOG = Logger
			.getLogger(UserServiceManager.class);
	/*
	 * This variable is used to hold the temp user.
	 */
	private UserTO tempUser;
	/*
	 * Indicate a new user.
	 */
	private boolean newUser;

	/**
	 * This is the hidden constructor so that we can make the class a
	 * singleton...
	 * 
	 */
	protected UserServiceManager() {
	}

	/**
	 * Method for returning an instance of this class.
	 * 
	 * @return
	 */
	public static UserServiceManager getInstance() {
		if (instance == null) {
			synchronized (UserServiceManager.class) {
				if (instance == null) {
					instance = new UserServiceManager();
				}
			}
		}
		return instance;
	}

	/**
	 * This is a helper method for returning the UserService.
	 * 
	 * @return
	 * @throws ServiceException
	 */
	private UserService getUserImplementation() throws ServiceException {
		// UserServiceService_Impl impl = new UserServiceService_Impl();
		// return impl.getUserService();
		// return new UserServiceServiceLocator().getUserService();

		return UserServiceRest.getInstance();

	}

	/**
	 * Method which will logon a user.
	 * 
	 * @param username
	 * @param password
	 * @return
	 */
	public UserTO logon(final String username, final String password) {
		// This is where the new implementation is located.
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().logon(username, password);
			}
		});
		LOG.info("LOGON return object = " + returnObj);
		if (returnObj instanceof UserTO) {
			// Got user now get roles on the user
			LOG.info("Returned user:" + ((UserTO) returnObj).getUsername());
			// Return user
			return (UserTO) returnObj;
		} else if (returnObj instanceof EntityNotFoundException) {
			LOG.error("ERR: (" + username + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M6);
			throw new RuntimeException("User not authenticated");
		} else if (returnObj instanceof UnauthorizedUserException) {
			LOG.error("ERR: (" + username + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M25);
			throw new RuntimeException("User not authorised");
		} else if (returnObj instanceof IllegalArgumentException) {
			LOG.error("ERR: (" + username + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M7);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + username + ") "
					+ ((Exception) returnObj).getMessage());
			return null;
		}
		return null;
	}

	/**
	 * This method will return the user by his username.
	 * 
	 * @param username
	 * @return
	 */
	public UserTO getUser(String username) {
		// This is where the new implementation is located.
		final String name = username;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().getUserByName(name);
			}
		});
		// The old way can stay the same except we need to check for timeout
		// exception also.
		if (returnObj instanceof UserTO) {
			// Got user now get roles on the user
			LOG.info("Returned user:" + ((UserTO) returnObj).getUsername());
			// Return user
			return (UserTO) returnObj;
		} else if (returnObj instanceof EntityNotFoundException) {
			LOG.error("ERR: (" + username + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M12);
			return null;
		} else if (returnObj instanceof IllegalArgumentException) {
			LOG.error("ERR: (" + username + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M23);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + username + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			SendErrorEmail.sendErrorMail((Exception) returnObj, "getUser",
					"UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This method will return a user based on its userid.
	 * 
	 * @param userid
	 * @return
	 */
	public UserTO getUser(long userid) {
		// This is where the new implementation is located.
		LOG.info("entering getUser");
		final long uid = userid;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().getUser(uid);
			}
		});
		if (returnObj instanceof UserTO) {
			// Got user now get roles on the user
			LOG.info("Returned user:" + ((UserTO) returnObj).getUsername());
			return (UserTO) returnObj;
		} else if (returnObj instanceof EntityNotFoundException) {
			LOG.error("ERR: (" + userid + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M12);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + userid + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			SendErrorEmail.sendErrorMail((Exception) returnObj, "getUser",
					"UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This method will create a user with its userroles.
	 * 
	 * @param u
	 * @param userRoles
	 * @return
	 */
	public UserTO createUser(final UserTO u) {
		LOG.info("Calling new UserServiceManager :" + u.toString());
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().createUser(u);
			}
		});
		if (returnObj instanceof UserTO) {
			LOG.info("Returned user:" + u.toString());
			return (UserTO) returnObj;
		} else if (returnObj instanceof DuplicateEntityException) {
			LOG.error("ERR: (" + u.toString() + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M1,
					new String[] { u.getUsername() });
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + u.toString() + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			SendErrorEmail.sendErrorMail((Exception) returnObj, "createUser",
					"UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This class will add roles to the user.
	 * 
	 * @param userid
	 * @param userroles
	 */
	/*
	 * public void saveUserRoles(UserTO u, RoleTO[] userRoles) { final UserTO
	 * userTO = u; final RoleTO roleTO[] = userRoles; LOG.info("Calling save
	 * user roles :" + u.toString()); for (int t = 0; t < roleTO.length; t++) {
	 * // Add roles final int i = t; if (roleTO[i] instanceof NurseTO) {
	 * executeServiceAction(new ServiceAction(new ServiceListener() { public
	 * Object performService() throws Exception {
	 * getUserImplementation().setNurseRole(userTO.getId().longValue(),
	 * (NurseTO)roleTO[i]); LOG.info("Setting nurse role"); return null; } }));
	 * } if (roleTO[i] instanceof DcTO) { executeServiceAction(new
	 * ServiceAction(new ServiceListener() { public Object performService()
	 * throws Exception {
	 * getUserImplementation().setDcRole(userTO.getId().longValue(),
	 * (DcTO)roleTO[i]); LOG.info("Setting dc role"); return null; } })); } if
	 * (roleTO[i] instanceof SysAdminTO) { executeServiceAction(new
	 * ServiceAction(new ServiceListener() { public Object performService()
	 * throws Exception { LOG.info("User is=" + userTO.getId());
	 * LOG.info("SysAdminRole=" + (SysAdminTO)roleTO[i]);
	 * getUserImplementation().setSysAdminRole(userTO.getId().longValue(),
	 * (SysAdminTO)roleTO[i]); LOG.info("Setting dc role"); return null; } }));
	 * } } }
	 */
	/**
	 * This method will update the user and its roles.
	 * 
	 * @param u
	 * @param userRoles
	 * @return
	 */
	public UserTO updateUser(final UserTO u) {
		LOG.info("Calling update user :" + u.toString());
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().updateUser(u);
			}
		});
		if (returnObj instanceof UserTO) {
			LOG.info("Returned user:" + u.toString());
			return u;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + u.toString() + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			SendErrorEmail.sendErrorMail((Exception) returnObj, "updateUser",
					"UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This method will get all the nurses.
	 * 
	 * @return UserTO[]
	 */
	public RoleTO[] getNurses() {
		LOG.info("Calling getNurses :");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().getAllNurses();
			}
		});
		if (returnObj instanceof RoleTO[]) {
			return (RoleTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: ( getNurses) "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			SendErrorEmail.sendErrorMail((Exception) returnObj, "getNurses",
					"UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This method will get all third party nurses.
	 * 
	 * @return UserTO[]
	 */
	public RoleTO[] getAllThirdPartyNurses() {
		LOG.info("Calling getAllThirdPartyNurses :");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().getAllThirdPartyNurses();
			}
		});
		if (returnObj instanceof RoleTO[]) {
			return (RoleTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: ( getAllThirdPartyNurses) "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getAllThirdPartyNurses", "UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This method will get all the nurses for a region
	 * 
	 * @return UserTO[]
	 */
	public RoleTO[] getNursesNySuburb(final SuburbTO sub) {
		LOG.info("Calling getNursesNySuburb :");
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().getNursesBySuburb(
						sub.getSuburbName());
			}
		});
		if (returnObj instanceof RoleTO[]) {
			return (RoleTO[]) returnObj;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: ( getNursesNySuburb) "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0);
			SendErrorEmail.sendErrorMail((Exception) returnObj,
					"getNursesNySuburb", "UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * This method finds all users with the given parameters.
	 * 
	 * @param surname
	 * @param firstname
	 * @return
	 */
	public UserTO[] getUsers(String surname, String firstname) {
		LOG.info("entering getUsers");
		final String sur = surname;
		final String first = firstname;
		Object returnObj = executeServiceAction(new ServiceAction() {
			public Object performService() throws Exception {
				return getUserImplementation().findUsers(sur, first);
			}
		});
		if (returnObj instanceof UserTO[]) {
			LOG.info("Returned user:" + surname);
			return (UserTO[]) returnObj;
		} else if (returnObj instanceof EntityNotFoundException) {
			LOG.error("ERR: (" + surname + " " + firstname + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M12);
			return null;
		} else if (returnObj instanceof Exception) {
			LOG.error("ERR: (" + surname + ") "
					+ ((Exception) returnObj).getMessage());
			ErrorMessages.showMessage(ErrorMessages.M0,
					new String[] { ((Exception) returnObj).getMessage() });
			SendErrorEmail.sendErrorMail((Exception) returnObj, "getUsers",
					"UserServiceManager");
			return null;
		}
		return null;
	}

	/**
	 * Method returns all the roles for a given user.
	 * 
	 * @param user
	 * @return
	 */
	/*
	 * public RoleTO[] getRoles(UserTO user) { LOG.info("entering getRoles for
	 * userid " + user.getId()); RoleTO[] roles; ArrayList rolArray = new
	 * ArrayList(); final UserTO userTO = user;
	 * 
	 * //Try to get the SystemAdmin role for user Object returnObj =
	 * executeServiceAction(new ServiceAction(new ServiceListener() { public
	 * Object performService() throws Exception { return
	 * getUserImplementation().getSysAdminByUserId(userTO.getId().longValue());
	 * } })); if (returnObj instanceof SysAdminTO) { rolArray.add(returnObj); }
	 * else if (returnObj instanceof EntityNotFoundException) { LOG.info("Role
	 * SysAdmin not
	 * found"); } else if (returnObj instanceof Exception) { LOG.error("ERR: ("
	 * + user.getUsername() + ") " + ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M0); }
	 * 
	 * //Try to get the DC role for user returnObj = executeServiceAction(new
	 * ServiceAction(new ServiceListener() { public Object performService()
	 * throws Exception { return
	 * getUserImplementation().getDcByUserId(userTO.getId().longValue()); } }));
	 * if (returnObj instanceof DcTO) { rolArray.add(returnObj); } else if
	 * (returnObj instanceof EntityNotFoundException) { LOG.info("Role DC not
	 * found"); } else if (returnObj instanceof Exception) { LOG.error("ERR: ("
	 * + user.getUsername() + ") " + ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M0); }
	 * 
	 * //Try to get the Nurse role for user returnObj = executeServiceAction(new
	 * ServiceAction(new ServiceListener() { public Object performService()
	 * throws Exception { return
	 * getUserImplementation().getNurseByUserId(userTO.getId().longValue()); }
	 * })); if (returnObj instanceof NurseTO) { rolArray.add(returnObj); } else
	 * if (returnObj instanceof EntityNotFoundException) { LOG.info("Role Nurse
	 * not found"); } else if (returnObj instanceof Exception) { LOG.error("ERR:
	 * (" + user.getUsername() + ") " + ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M0); } // Set up roles roles =
	 * new RoleTO[rolArray.size()]; // int x = 0; // for (Iterator i =
	 * rolArray.iterator(); i.hasNext();) { // roles[x] = (RoleTO) i.next(); //
	 * x++; // } rolArray.toArray(roles); return roles; }
	 */
	/**
	 * Get the DC Role by id.
	 * 
	 * @param userid
	 * @return
	 */
	/*
	 * public DcTO getDc(long dcid) { LOG.info("entering getDc"); final long id
	 * = dcid;
	 * 
	 * Object returnObj = executeServiceAction(new ServiceAction(new
	 * ServiceListener() { public Object performService() throws Exception {
	 * return getUserImplementation().getDcById(id); } }));
	 * 
	 * if (returnObj instanceof DcTO) { LOG.info("Returned user:" + ((DcTO)
	 * returnObj).getId()); return (DcTO) returnObj; } else if (returnObj
	 * instanceof EntityNotFoundException) { LOG.error("ERR: (" + dcid + ") "+
	 * ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M12); return null; } else if
	 * (returnObj instanceof Exception) { LOG.error("ERR: (" + dcid + ") " +
	 * ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M0); return null; } return null;
	 * }
	 */
	/**
	 * Get the Nurse Role by id.
	 * 
	 * @param userid
	 * @return
	 */
	/*
	 * public NurseTO getNurse(long nurseid) { LOG.info("entering getNurse");
	 * final long id = nurseid;
	 * 
	 * Object returnObj = executeServiceAction(new ServiceAction(new
	 * ServiceListener() { public Object performService() throws Exception {
	 * return getUserImplementation().getNurseById(id); } }));
	 * 
	 * if (returnObj instanceof NurseTO) { LOG.info("Returned Nurse:" +
	 * ((NurseTO) returnObj).getId()); return (NurseTO) returnObj; } else if
	 * (returnObj instanceof EntityNotFoundException) { LOG.error("ERR: (" + id
	 * + ") "+ ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M12); return null; } else if
	 * (returnObj instanceof Exception) { LOG.error("ERR: (" + id + ") " +
	 * ((Exception) returnObj).getMessage());
	 * ErrorMessages.showMessage(ErrorMessages.M0); return null; } return null;
	 * }
	 */
	/**
	 * This method will return the appointments for a nurse.
	 * 
	 * @param n
	 * @return
	 */
	public List getAppointmentForNurse(UserTO n) {
		Map appointments = new HashMap();
		return (List) appointments.get(n);
	}

	public UserTO getTempUser() {
		return tempUser;
	}

	public void setTempUser(UserTO tempUser) {
		this.tempUser = tempUser;
	}

	public boolean isNewUser() {
		return newUser;
	}

	public void setNewUser(boolean newUser) {
		this.newUser = newUser;
	}
}
